from django.contrib import admin
from django.contrib.contenttypes.admin import (GenericStackedInline,
    GenericTabularInline)
from django.db.models import Count

from taggit.models import TaggedItem


class TaggitCounter():
    """
    Display (and sort by) number of Taggit tags associated with tagged items.

    Usage Example:
        from taggit_helpers import TaggitCounter

        class MyModelAdmin(TaggitCounter, admin.ModelAdmin):
            list_display = (
                ...
                'taggit_count',
            )

    Note: Currently, the TaggableManager() field must be named 'tags'.

    Note: To avoid overcounting, set distinct=True if further annotating the queryset with Count():

        queryset.annotate(m2m_field_count=Count('m2m_field', distinct=True))
    """

    def queryset(self, request):
        queryset = self.model.objects.get_query_set()
        return queryset.annotate(taggit_count=Count('tags', distinct=True))

    def taggit_count(self, obj):
        return obj.taggit_count
    taggit_count.admin_order_field = 'taggit_count'
    taggit_count.short_description = '# of Tags'


class TaggitListFilter(admin.SimpleListFilter):
    """
    Filter records by Taggit tags for the current model only.
    Tags are sorted alphabetically by name.

    Usage Example:
        from taggit_helpers import TaggitListFilter

        class MyModelAdmin(admin.ModelAdmin):
            list_filter = [TaggitListFilter]
    """

    title = 'Tags'
    parameter_name = 'tag'

    def lookups(self, request, model_admin):
        model_tags = [tag.name for tag in
            TaggedItem.tags_for(model_admin.model)]
        model_tags.sort()
        return tuple([(tag, tag) for tag in model_tags])

    def queryset(self, request, queryset):
        if self.value() is not None:
            return queryset.filter(tags__name=self.value())


class TaggitInlineBase():
    """
    Base model for Taggit admin inlines.
    Use TaggitStackedInline or TaggitTabularInline.
    """
    model = TaggedItem
    verbose_name = 'Tag'
    verbose_name_plural = 'Tags'
    ordering = ('tag__name',)


class TaggitStackedInline(TaggitInlineBase, GenericStackedInline):
    """
    Add stacked inline for Taggit tags to admin.
    Tags are sorted alphabetically by name.

    Usage example:
        from taggit_helpers import TaggitStackedInline

        class MyModelAdmin(admin.ModelAdmin):
            inlines = [TaggitStackedInline]
    """
    pass


class TaggitTabularInline(TaggitInlineBase, GenericTabularInline):
    """
    Add tabular inline for Taggit tags to admin.
    Tags are sorted alphabetically by name.

    Usage example:
        from taggit_helpers import TaggitTabularInline

        class MyModelAdmin(admin.ModelAdmin):
            inlines = [TaggitTabularInline]
    """
    pass



import os
import sys
from setuptools import setup

if sys.version_info < (3, 2):
    print("Sorry, django-taggit-helpers currently requires Python 3.2+.")
    sys.exit(1)

# From: https://hynek.me/articles/sharing-your-labor-of-love-pypi-quick-and-dirty/
def read(*paths):
    """Build a file path from *paths* and return the contents."""
    with open(os.path.join(*paths), 'r') as f:
        return f.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

install_requires = [
    'django>=1.7,<1.8',
    'django-taggit>=0.12.0',
]

setup(
    name='django-taggit-helpers',
    version='0.1.0',
    packages=['taggit_helpers'],
    include_package_data=True,
    license='BSD License',
    keywords='tags tagging taggit django-taggit',
    description='Django admin helper classes for django-taggit tags',
    long_description=(read('README.rst') + '\n\n' +
                      read('CHANGELOG.rst')),
    url='https://github.com/mfcovington/django-taggit-helpers',
    author='Michael F. Covington',
    author_email='mfcovington@gmail.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.7',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3 :: Only',
    ],
    install_requires=install_requires,
)
